import logging
import re

import lxml.etree as ET


_account_alias_pattern = re.compile("Account: ([^(]+) \(([0-9]+)\)")


def account_aliases(session, username, password, auth_method, saml_response, config):
    alias_response = session.post(
        'https://signin.aws.amazon.com/saml',
        verify=config.ssl_verification,
        headers={
            'Accept-Language': 'en',
            'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko',
            'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
            'Accept': 'text/plain, */*; q=0.01',
        },
        auth=None,
        data={
            'UserName': username,
            'Password': password,
            'AuthMethod': auth_method,
            'SAMLResponse': saml_response,
        }
    )

    logging.debug(u'''Request:
        * url: {}
        * headers: {}
    Response:
        * status: {}
        * headers: {}
        * body: {}
    '''.format('https://signin.aws.amazon.com/saml',
               alias_response.request.headers,
               alias_response.status_code,
               alias_response.headers,
               alias_response.text))

    html_response = ET.fromstring(alias_response.text, ET.HTMLParser())

    accounts = {}
    account_element_query = './/div[@class="saml-account-name"]'
    for account_element in html_response.iterfind(account_element_query):
        m = _account_alias_pattern.search(account_element.text)
        if m is not None:
            accounts[m.group(2)] = m.group(1)

    return accounts



from aws_adfs import account_aliases_fetcher


def _aws_account(account_alias, account_no):
    return u'<div class="saml-account-name">Account: {} ({})</div>'.format(account_alias, account_no)


def _account_page_response(accounts):
    response = type('', (), {})()
    response.request = type('', (), {})()
    response.request.headers = {}
    response.status_code = 'irrelevant'
    response.headers = {}
    response.text = u'''
    <html>
        <body>
            <div>
                <form>
                    <fieldset>
                        {}
                    </fieldset>
                </form>
            </div>
        </body>
    </html>
    '''.format('\n'.join([account for account in accounts]))

    return response


class TestAccountAliasesFetcher:
    
    def test_returns_empty_account_dictionary_when_no_account_are_named(self):
        # given user with no aws accounts
        self.authenticated_session.post = lambda *args, **kwargs: _account_page_response([])

        # when gets account aliases via fetcher
        accounts = account_aliases_fetcher.account_aliases(self.authenticated_session,
                                                           self.irrelevant_username,
                                                           self.irrelevant_password,
                                                           self.irrelevant_auth_method,
                                                           self.authenticated_saml_response,
                                                           self.irrelevant_config)
        # then returns no accounts
        assert accounts == {}

    def test_returns_one_account_when_one_account_is_listed(self):
        # given user with no aws accounts
        account_no = '123'
        account_alias = 'single'
        self.authenticated_session.post = lambda *args, **kwargs: _account_page_response([_aws_account(account_alias, account_no)])

        # when gets account aliases via fetcher
        accounts = account_aliases_fetcher.account_aliases(self.authenticated_session,
                                                           self.irrelevant_username,
                                                           self.irrelevant_password,
                                                           self.irrelevant_auth_method,
                                                           self.authenticated_saml_response,
                                                           self.irrelevant_config)
        # then returns no accounts
        assert accounts == {account_no: account_alias}

    def test_returns_two_accounts_when_two_accounts_are_listed(self):
        # given user with no aws accounts
        account_no = '1'
        account_alias = 'single'
        second_account_no = '2'
        second_account_alias = 'bingle'
        self.authenticated_session.post = lambda *args, **kwargs: _account_page_response([
            _aws_account(account_alias, account_no),
            _aws_account(second_account_alias, second_account_no),
        ])

        # when gets account aliases via fetcher
        accounts = account_aliases_fetcher.account_aliases(self.authenticated_session,
                                                           self.irrelevant_username,
                                                           self.irrelevant_password,
                                                           self.irrelevant_auth_method,
                                                           self.authenticated_saml_response,
                                                           self.irrelevant_config)
        # then returns no accounts
        assert accounts == {account_no: account_alias, second_account_no: second_account_alias}

    def setup_method(self, method):
        self.authenticated_session = type('', (), {})()
        self.irrelevant_auth_method = {}
        self.irrelevant_username = 'irrelevant username'
        self.irrelevant_password = 'irrelevant password'
        self.authenticated_saml_response = 'irrelevant saml response'
        self.irrelevant_config = type('', (), {})()
        self.irrelevant_config.ssl_verification = True
